from django.contrib.auth.signals import user_logged_in, user_logged_out
from django.contrib.sessions.models import Session
from django.db.models.signals import post_delete

from .models import UserSession


def user_logged_in_handler(sender, request, user, **kwargs):
    user_session, created = UserSession.objects.get_or_create(
        user=user,
        session_key=request.session.session_key
    )
    if not user.is_superuser:
        sessions = UserSession.objects.filter(user=user) \
                            .exclude(pk=user_session.pk)
        sessions.delete()

user_logged_in.connect(user_logged_in_handler)


def user_logged_out_handler(sender, request, user, **kwargs):
    try:
        user_session = UserSession.objects.get(
            user=user,
            session_key=request.session.session_key
        )
        user_session.delete()
    except UserSession.DoesNotExist:
        pass

user_logged_out.connect(user_logged_out_handler)


def user_session_delete_handler(sender, instance, using, **kwargs):
    if isinstance(instance, UserSession):
        try:
            session = Session.objects.get(session_key=instance.session_key)
            session.delete()
        except Session.DoesNotExist:
            pass

post_delete.connect(user_session_delete_handler)



from django.apps import AppConfig


class DJSessionsConfig(AppConfig):
    name = 'djsessions'
    verbose_name = 'djsessions'

    def ready(self):
        from .signals import user_logged_in  # noqa
        from .signals import user_logged_out  # noqa
        from .signals import post_delete  # noqa



from django.conf import settings
from django.contrib.sessions.models import Session
from django.utils.timezone import now, timedelta

from .models import UserSession

DJ_CHANNELS_EXPIRE_DEFAULT = 12 * 3600  # 12 hours
DJ_CHANNELS_EXPIRE = getattr(
    settings,
    "DJ_CHANNELS_EXPIRE",
    DJ_CHANNELS_EXPIRE_DEFAULT
)


def remove_old_channel_sessions():
    """ Channel sessions are removed because these are set by websockets, and
        are recreated any time there is a socket disconnect (latency, server restart, browser refresh, etc). This assumes that the channel session is
        being set by the HTTP browser.

        :return int: Number of records removed.
    """
    sessions = Session.objects.filter(session_key__startswith='chn')
    sessions = sessions.filter(expire_date__lt=now())
    removed, data = sessions.delete()
    return removed

def remove_orphaned_sessions():
    """ Orphaned sessions are removed because sessions can be created multiple
        times per UserSession. This happens whenever there is a socket
        disconnect (latency, server restart, browser refresh, etc). This
        assumes that the channel session is being set by the HTTP browser.

        :return int: Number of records removed.
    """
    keys = UserSession.objects.values_list('session_key', flat=True)
    sessions = Session.objects.exclude(session_key__in=keys)
    removed, data = sessions.delete()
    return removed



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-07-06 21:14
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='UserSession',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('session_key', models.CharField(max_length=40, unique=True, verbose_name='session key')),
                ('user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# -*- coding: utf-8 -*-
from django.conf import settings
from django.contrib.sessions.models import Session
from django.db import models

from model_utils.models import TimeStampedModel


class UserSession(TimeStampedModel):
    user = models.ForeignKey(settings.AUTH_USER_MODEL, null=True, on_delete=models.SET_NULL)
    # session = models.ForeignKey(Session, null=True, on_delete=models.SET_NULL)
    session_key = models.CharField('session key', max_length=40, unique=True)

    def __str__(self):
        return self.user.username



from django.contrib.sessions.models import Session
from django.core.management.base import BaseCommand

from ...utils import (
    remove_old_channel_sessions,
    remove_orphaned_sessions
)


class Command(BaseCommand):
    help = 'Cleans the database of expired records'

    def handle(self, *args, **options):

        # Remove old channel sessions
        removed = remove_old_channel_sessions()
        msg = "Removed {} expired channel sessions.".format(removed)
        self.stdout.write(msg)

        # Remove old channel sessions
        removed = remove_orphaned_sessions()
        msg = "Removed {} orphaned channel sessions.".format(removed)
        self.stdout.write(msg)

        msg = "{} sessions remain".format(Session.objects.count())
        self.stdout.write(msg)
